/*
 *  
 *  Copyright (c) by Shuu Yamaguchi <shuu@wonder.ne.jp>
 *
 *  Can be freely distributed and used under the terms of the GNU GPL.
 */
#include	<stdio.h>

#include	"usbmgr.h"

#define	CONFIG_TYPE_NONE	0
#define	CONFIG_TYPE_VENDOR	1
#define	CONFIG_TYPE_CLASS	2

#define	CONFIG_ID_INVALID	0x00

static struct node *config_ring = NULL;

/*
 * create config
 */
struct config *
create_config(void)
{
	struct config *conf;

	conf = (struct config *)malloc(sizeof(struct config));
	if (conf == NULL)
		return NULL;
	init_config(conf);

	return conf;
}

/*
 * initialize config
 */
void
init_config(struct config *conf)
{
	INIT_NODE(&(conf->link));
	SET_TYPE_CONFIG(&(conf->link));
	conf->id.vendor = conf->id.product = CONFIG_ID_INVALID;
	conf->id.class = conf->id.subclass = conf->id.protocol = CONFIG_ID_INVALID;
	conf->id.type = CONFIG_TYPE_NONE;
	conf->script = NULL;
}

/*
 * link to config_ring
 */
void
link_config(struct config *conf)
{
	link_node(&config_ring,&(conf->link));
}

/*
 *  validate type in config
 */
void
validate_config(struct config *conf)
{
	conf->id.type = CONFIG_TYPE_NONE;
	if (conf->id.vendor != CONFIG_ID_INVALID ||
		conf->id.product != CONFIG_ID_INVALID)
		conf->id.type |= CONFIG_TYPE_VENDOR;
	if (conf->id.class != CONFIG_ID_INVALID ||
		conf->id.subclass != CONFIG_ID_INVALID ||
		conf->id.protocol != CONFIG_ID_INVALID)
		conf->id.type |= CONFIG_TYPE_CLASS;
}

/*
 * match config
 * [return]
 *	1: match   0: not match
 */
#define	CONFIG_MATCH	1
#define	CONFIG_NOTMATCH	0
int
match_config(struct config conf1,struct config conf2)
{
	int conf_type;

	conf_type = conf1.id.type & conf2.id.type;
	if (!conf_type)
		return CONFIG_NOTMATCH;	/* boo */
	if (conf_type & CONFIG_TYPE_VENDOR) {
		if (conf1.id.vendor == conf2.id.vendor &&
			conf1.id.product == conf2.id.product)
			return CONFIG_MATCH;	/* match */
		else
			return CONFIG_NOTMATCH;
	} else {	/* CONFIG_TYPE_CLASS */
		if (conf1.id.class == conf2.id.class &&
			conf1.id.subclass == conf2.id.subclass &&
			conf1.id.protocol == conf2.id.protocol)
			return CONFIG_MATCH;
		else
			return CONFIG_NOTMATCH;
	}
	return 0;	/* not reached */
}


/*
 * search config
 */
struct config *
find_config(struct config *start,struct config target)
{
	struct config *conf;

	for(conf = start; ;conf = (struct config *)conf->link.next) {
		if (match_config(*conf,target))	/* match */
			return conf;
		if (conf->link.next == (struct node *)start)
			break;
	}
	return NULL;
}

/*
 * search config in config_ring
 */
struct config *
find_config_all(struct config target)
{
	return find_config((struct config *)config_ring,target);
}

/*
 * dump config data from config_ring
 */
#ifdef	DEBUG
void
dump_configtree_ring(void)
{
	struct config *conf;

	for(conf = (struct config *)config_ring; ;conf = (struct config *)conf->link.next) {
		syslog(LOG_DEBUG,"config:%p <- %p(modlink:%p,script:\"%s\") -> %p", 
			conf->link.prev,conf,conf->link.data,conf->script,conf->link.next);
		dump_modlink(conf->link.data);
		if (config_ring == conf->link.next)
			break;
	}
}
#endif	/* DEBUG */
