/*
 *  
 *  Copyright (c) by Shuu Yamaguchi <shuu@wonder.ne.jp>
 *
 *  Can be freely distributed and used under the terms of the GNU GPL.
 */
#include	<stdio.h>
#include	<string.h>

#include	"usbmgr.h"

static struct node *device_ring = NULL;

/*
 * create usbdevice
 */
struct usbdevice *
create_device(struct node *parent)
{
	struct usbdevice *dev;

	dev = (struct usbdevice *)malloc(sizeof(struct usbdevice));
	if (dev == NULL)
		return NULL;
	INIT_NODE(&(dev->link));
	SET_TYPE_DEVICE(&(dev->link));
	dev->pdev = parent;
	init_config(&(dev->conf));
	link_node(&device_ring,&(dev->link));

	return dev;
}

/*
 * search device using config
 */
struct usbdevice *
find_device(struct usbdevice *start,struct usbdevice target)
{
	struct usbdevice *dev;

	if (start == NULL)
		return NULL;
	for(dev = start; ;dev = (struct usbdevice *)dev->link.next) {
		if (match_config(dev->conf,target.conf))	/* match */
			return dev;
		if (dev->link.next == (struct node *)start)
			break;
	}
	return NULL;
}

struct usbdevice *
find_device_all(struct usbdevice target)
{
	return find_device((struct usbdevice *)device_ring,target);
}

/*
 * search module named "name" ring
 * --- recursive ---
 */
struct node *
find_string_device(struct usbdevice *start, char *name)
{
	struct usbdevice *dev;
	struct node *module;

	if (start == NULL)		/* device_ring etc */
		return NULL;
	for (dev = start; ;dev = (struct usbdevice *)dev->link.next) {
		/* child type == usbdevice -- recursive -- */
		if (NODE_IS_DEVICE(*(start->link.data)) &&
			(module = find_string_device((struct usbdevice *)dev->link.data,name)) != NULL) {
			return module;
		}
		/* child type == modlink */
		if ((module = find_string_modlink(start->link.data,name)) != NULL)
				return module;
		if (dev->link.next == (struct node *)start)
			break;
	}
	return NULL;
}

struct node *
find_string_device_all(char *name)
{
	return find_string_device((struct usbdevice *)device_ring, name);
}
