/*
 *  
 *  Copyright (c) by Shuu Yamaguchi <shuu@wonder.ne.jp>
 *
 *  Can be freely distributed and used under the terms of the GNU GPL.
 */
#include	<unistd.h>
#include	<sys/types.h>
#include	<fcntl.h>
#include	<limits.h>
#include	<string.h>

#include	"usbmgr.h"

#define	MODPROBLE_FILE "/proc/sys/kernel/modprobe"

static char *module_loader = NULL;

int
get_loader(void)
{
	int fd;
	char ld[PATH_MAX+1];
	ssize_t len;

	if ((fd = open(MODPROBLE_FILE,O_RDONLY)) == -1)
		return -1;
	if ((len = read(fd,ld,PATH_MAX)) <= 0)
		return -1;
	close(fd);
	if (ld[len-1] == '\n')
		ld[len-1] = '\0';
	else
		ld[len] = '\0';
	if((module_loader = strdup(ld)) == NULL)
		return -1;

	return 0;
}

	
static void
load(char *loader,char *name)
{
	pid_t pid;
	char *arg[3];

	pid = fork();
	switch(pid) {
	case -1:
		syslog(LOG_ERR,"fork() error");
		break;
	case 0:	/* child */
		arg[0] = loader;
		arg[1] = name;
		arg[2] = NULL;
		if (debug)
			syslog(LOG_DEBUG,"execv\"%s\":\"%s\",\"%s\",\"%s\"\n",
				loader,arg[0],arg[1],arg[2]);

		execv(loader,arg);
		syslog(LOG_ERR,"execv() error");
		break;
	default:	/* parent */
		wait(NULL);
		break;
	}
}

/*
 * load module
 * USB_NOMODULE is not load
 */
int 
module_load(char *name)
{
	if (debug)
		syslog(LOG_DEBUG,"load module \"%s\"",name);
	if (!strcmp(name,USB_NOMODULE)) {
		if (debug)
			syslog(LOG_DEBUG,"not load module \"%s\"",name);
		return 0;
	}
	if (module_loader == NULL) {
		if (get_loader() == -1)	{
			syslog(LOG_ERR,"Can't get module loader");
			return -1;
		}
	}

	load(module_loader,name);

	return 0;
}

/*
 * execute script in struct config
 */

void
execute_script(char *cmd)
{
	pid_t pid;
	char *arg[2];
	char path[PATH_MAX+1];

	if (strlen(cmd) + sizeof(USB_DIR_ETC) + 1 > PATH_MAX) {
		syslog(LOG_ERR,"script path is too long");
		return;
	}
	sprintf(path,"%s/%s",USB_DIR_ETC,cmd);

	pid = fork();
	switch(pid) {
	case -1:
		syslog(LOG_ERR,"fork() error");
		break;
	case 0:	/* child */
		arg[0] = cmd;
		arg[1] = NULL;
		if (debug)
			syslog(LOG_DEBUG,"execv\"%s\":\"%s\",\"%s\"\n",
				path,arg[0],arg[1]);
		execv(path,arg);
		syslog(LOG_ERR,"execv() error");
		break;
	default:	/* parent */
		wait(NULL);
		break;
	}
}
