/*
 *  usbmgr  --- USB Manager
 *  
 *  Copyright (c) by Shuu Yamaguchi <shuu@wonder.ne.jp>
 *
 *  Can be freely distributed and used under the terms of the GNU GPL.
 */
#include	<stdio.h>
#include	<unistd.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<string.h>
#include	<signal.h>
#include	<dirent.h>
#include	<limits.h>
#include	<ctype.h>

#include	"usbmgr.h"
#include	"version.h"

#define	PID_FILE	USB_DIR_PID "/" USB_FILE_PID

static char *conf_file = CONF_PATH;

/****** option flag ******/
int nodaemon = 0;
int debug = 0;
static unsigned int sleep_time = SLEEP_TIME;

#ifdef	MULTIPLE_USBBUS
static void
observe(void)
{
	DIR *dp;
	struct dirent *de;
	char path[PATH_MAX+1];

	if ((dp = opendir(USB_PROCDIR)) == NULL) {
		syslog(LOG_ERR,"Can't open directory \"%s\"",USB_PROCDIR);
		return;
	}
	while((de = readdir(dp)) != NULL) {
		if (strlen(de->d_name) == 3 && isdigit(de->d_name[0]) &&
			isdigit(de->d_name[1]) && isdigit(de->d_name[2])) {
			if (!strcmp(de->d_name,"001"))
				goto loop_end;
			switch(fork()) {
			case 0:		/* child */
				break;
			case -1:	/* error */
				syslog(LOG_ERR,"fork error");
				return;
			default:	/* parent */
				goto loop_end;
				break;
			}
		}
	}
	closedir(dp);
	return;
loop_end:
	sprintf(path,"%s/%s",USB_PROCDIR,de->d_name);
	while(1) {
		observ_proc_nnn(path);
		sleep(sleep_time);
	}
}
#else
static void
observe(void)
{
	while(1) {
		observ_proc_nnn(USB_PROCDIR "/001");
		sleep(sleep_time);
	}
}
#endif

static void
daemonize(void)
{
	switch(fork()) {
	case 0:		/* child */
		break;	/* do nothing */
	case -1:	/* error */
		exit(1);
	default:	/* parent */
		exit(0);
	}
}

static int
create_pidfile()
{
	int fd,len;
	char pid_buf[10];	/* 8 + '\n' + '\0' */

	/* todo: if pid file exist, print warning */
	if((fd = creat(PID_FILE,0644)) == -1)
		return -1;
	sprintf(pid_buf,"%d\n",getpid());
	len = strlen(pid_buf);
	if(write(fd,pid_buf,len) != len)
		return -1;
	close(fd);
	return 0;
}

static void
func_hup(int dummy)
{
	load_config(conf_file);
}

static void
init_signal(void)
{
	struct sigaction sa;

	sa.sa_handler = func_hup;
	sigemptyset(&(sa.sa_mask));
	sa.sa_flags = SA_RESTART;
	sigaction(SIGHUP,&sa,NULL);
}

int
main(int argc,char **argv)
{
	int opt;

	while((opt = getopt(argc,argv,"D:Vc:dnt:")) != EOF) {
		switch(opt) {
		case 'D':
			debug = (unsigned int)strtoul(optarg,NULL,0);
			break;
		case 'V':
			printf("usbmgr: version %s\n",usbmgr_version);
			exit(0);
		case 'c':
			conf_file = optarg;
			break;
		case 'd':
			debug++;
			nodaemon = 1;
			break;
		case 'n':
			nodaemon = 1;
			break;
		case 't':
			sleep_time = (unsigned int)strtoul(optarg,NULL,0);
			break;
		}
	}
#ifdef	DEBUG
	openlog("usbmgr",LOG_PID|LOG_PERROR,LOG_DAEMON);
#else
	openlog("usbmgr",LOG_PID|LOG_CONS,LOG_DAEMON);
	close(0);
	close(1);
	close(2);
#endif
	syslog(LOG_INFO,"start %s",usbmgr_version);
	load_config(conf_file);
#ifdef	DEBUG
	if (debug) {
		dump_moduletree_ring();
		dump_configtree_ring();
	}
#endif
	if (!nodaemon) {
		daemonize();
	}
	if (create_pidfile() == -1) {
		syslog(LOG_ERR,"Cant create pid file");
		exit(1);
	}
	init_signal();
	observe();
}
