/*
 *  
 *  Copyright (c) by Shuu Yamaguchi <shuu@wonder.ne.jp>
 *
 *  Can be freely distributed and used under the terms of the GNU GPL.
 */
#include	<stdio.h>	/* TODO:delete          for stderr */
#include	<unistd.h>
#include	<sys/stat.h>
#include	<sys/mman.h>
#include	<ctype.h>
#include	<fcntl.h>
#include	<string.h>

#include	"usbmgr.h"

/* token in config file */
#define	USB_HOST		"host"
#define	USB_VENDOR		"vendor"
#define	USB_PRODUCT		"product"
#define	USB_CLASS		"class"
#define	USB_SUBCLASS	"subclass"
#define	USB_PROTOCOL	"protocol"
#define	USB_MODULE		"module"	/* must be last token in statement */
#define	USB_CONNECT		","			/* must be one character */
#define	USB_SCRIPT		"script"

#define	USB_HOST_SZ		(sizeof(USB_HOST)-1)
#define	USB_VENDOR_SZ	(sizeof(USB_VENDOR)-1)
#define	USB_PRODUCT_SZ	(sizeof(USB_PRODUCT)-1)
#define	USB_CLASS_SZ	(sizeof(USB_CLASS)-1)
#define	USB_SUBCLASS_SZ	(sizeof(USB_SUBCLASS)-1)
#define	USB_PROTOCOL_SZ	(sizeof(USB_PROTOCOL)-1)
#define	USB_MODULE_SZ	(sizeof(USB_MODULE)-1)
#define	USB_CONNECT_SZ	(sizeof(USB_CONNECT)-1)
#define	USB_SCRIPT_SZ	(sizeof(USB_SCRIPT)-1)

#define	USB_HOST_ID		0
#define	USB_VENDOR_ID	1
#define	USB_PRODUCT_ID	2
#define	USB_CLASS_ID	3
#define	USB_SUBCLASS_ID	4
#define	USB_PROTOCOL_ID	5
#define	USB_MODULE_ID	6
#define	USB_CONNECT_ID	7
#define	USB_SCRIPT_ID	8

struct config_token {
	char *name;
	int size;
};

static struct config_token token[] = {
	{USB_HOST,USB_HOST_SZ},
	{USB_VENDOR,USB_VENDOR_SZ},
	{USB_PRODUCT,USB_PRODUCT_SZ},
	{USB_CLASS,USB_CLASS_SZ},
	{USB_SUBCLASS,USB_SUBCLASS_SZ},
	{USB_PROTOCOL,USB_PROTOCOL_SZ},
	{USB_MODULE,USB_MODULE_SZ},
	{USB_CONNECT,USB_CONNECT_SZ},
	{USB_SCRIPT,USB_SCRIPT_SZ},
	{NULL,-1}
};

/* points to the position which is invalid statement in configuration file */
static char *error_ptr;		
char *host_module;

/*
 * search a token
 *
 * [return]
 *	index from the founded token[]
 *  -1: find no token
 *   *: index of token[]
 */ 
int
find_token(char *ptr,char *endp)
{
	int i,left;

	left = endp - ptr;
	if (left <= 0)
		return -1;
	if (*ptr == *USB_CONNECT)		/* compare characters */
		return USB_CONNECT_ID;
	for(i = 0;token[i].name != NULL;i++) {
		if(left >= token[i].size &&
			!strncmp(ptr, token[i].name, token[i].size) &&
			isspace(*(ptr+token[i].size)))
			return i;
	}
	return -1;
}

/*
 * [return]
 * 	length of token which have no '\0'
 *  0: no token
 */
int
find_arg(char *ptr,char *endp)
{
	char *ptr_orig;

	ptr_orig = ptr;
	for(;ptr < endp;ptr++) {
		if (isspace(*ptr) || *ptr == *USB_CONNECT)	/* arg1,<any> */
			return ptr - ptr_orig;
	}
	return 0;
}

#define		ST_NEW		1		/* new statement */
#define		ST_END		2		/* statement end */

int
parse_token(char *ptr,char *endp)
{
	int id,prev_id;
	int get_arg_flag = 0;
	int status = ST_END;
	struct config *conf;

	while(ptr < endp) {
		/* delete white space */
		if(isspace(*ptr)) {
			ptr++;
			continue;
		}
		/* delete comments */
		if (*ptr == '#') {
			while(ptr < endp) {
				if (*ptr == '\n')
					break;
				ptr++;
			}
			continue;
		}
		if (!get_arg_flag) {
			if ((id = find_token(ptr,endp)) == -1) {
				error_ptr = ptr;
				return -1;
			}
			if (debug > 1)
				syslog(LOG_DEBUG,"real token \"%s\"",token[id].name);
			get_arg_flag = 1;
			ptr += token[id].size;
			if (id == USB_CONNECT_ID) {
				if (prev_id != USB_MODULE_ID)
					return -1;
				id = prev_id;
			}
			if (id == USB_MODULE_ID) {	/* config statement end ? */
				status = ST_END;
			} else {
				if(status == ST_END) {
					conf = create_config();
					if (conf == NULL)
						return -1;
					link_config(conf);	/* link to config_ring */
				}
				status = ST_NEW;
			}
			if (debug > 1)
				syslog(LOG_DEBUG, "token \"%s\"",token[id].name);
		} else {	/* get argument of the founded token */
			int len;
			char *arg;
			struct node *mod;

			if ((len = find_arg(ptr,endp)) == 0) {
				error_ptr = ptr;
				return -1;	/* no argument */
			}
			if ((arg = (char *)malloc(len+1)) == NULL)
				return -1;
			strncpy(arg,ptr,len);
			arg[len] = '\0';
			if (debug > 1)
				syslog(LOG_DEBUG,"arg \"%s\"",arg);
			switch(id) {
			case USB_HOST_ID:
				if ((host_module = strdup(arg)) == NULL)
					return -1;
				break;
			case USB_VENDOR_ID:
				conf->id.vendor = (__uint16_t)strtoul(arg,NULL,0);
				break;
			case USB_PRODUCT_ID:
				conf->id.product = (__uint16_t)strtoul(arg,NULL,0);
				break;
			case USB_CLASS_ID:
				conf->id.class = (__uint8_t)strtoul(arg,NULL,0);
				break;
			case USB_SUBCLASS_ID:
				conf->id.subclass = (__uint8_t)strtoul(arg,NULL,0);
				break;
			case USB_PROTOCOL_ID:
				conf->id.protocol = (__uint8_t)strtoul(arg,NULL,0);
				break;
			case USB_MODULE_ID:	/* must be last token */
				validate_config(conf);	/* setup id data */
				if((mod = find_string_module(arg)) == NULL) {
					mod = create_module(arg);
					if (mod == NULL)
						return -1;
				}
				/* config link to module */
				if (create_modlink(&(conf->link.data),mod) == NULL)
					return -1;
				break;
			case USB_SCRIPT_ID:
				if((conf->script = strdup(arg)) == NULL)
					return -1;
				break;
			}
			free(arg);
			prev_id = id;
			ptr += len;
			get_arg_flag = 0;
		}
	}	
}

/*
 * Function: load_config
 * [input]
 *	fname: configuration file name
 * [output]
 *   0: success
 *	-1: error
 */
int
load_config(char *fname)
{
	struct stat st;
	int fd;
	char *mp;

	if ((fd = open(fname,O_RDONLY)) == -1)
		return -1;
	if(fstat(fd,&st) == -1)
		return -1;
	
	if((mp = (char *)mmap(0,st.st_size,PROT_READ,MAP_SHARED,fd,0)) == MAP_FAILED)
		return -1;
	close(fd);

	if(parse_token(mp, mp+st.st_size) == -1) {
		if (error_ptr != NULL) {
			syslog(LOG_ERR,"Syntax error near %d bytes offset in configuration file",error_ptr - mp);
		}
		return -1;
	}

	return munmap(mp,st.st_size);
}

