/*
 *  
 *  Copyright (c) by Shuu Yamaguchi <shuu@wonder.ne.jp>
 *
 *  Can be freely distributed and used under the terms of the GNU GPL.
 */
#include	<stdio.h>
#include	<sys/types.h>
#include	<sys/stat.h>
#include	<fcntl.h>
#include	<dirent.h>
#include	<ctype.h>
#include	<sys/mount.h>
#include	<limits.h>

#include	"usbmgr.h"


#if 0
#include	<linux/usb.h>
#else
typedef unsigned char __u8;
typedef unsigned short __u16;

struct usb_device_descriptor {
	__u8  bLength;
	__u8  bDescriptorType;
	__u16 bcdUSB;
	__u8  bDeviceClass;
	__u8  bDeviceSubClass;
	__u8  bDeviceProtocol;
	__u8  bMaxPacketSize0;
	__u16 idVendor;
	__u16 idProduct;
	__u16 bcdDevice;
	__u8  iManufacturer;
	__u8  iProduct;
	__u8  iSerialNumber;
	__u8  bNumConfigurations;
};
#endif

/*
 * mount -t usbdevfs /proc/bus/usb /proc/bus/usb
 */
void
do_mount(void)
{
	char *arg[] = {"mount","-t","usbdevfs",USB_PROCDIR,USB_PROCDIR,NULL};
	switch(fork()){
	case 0:		/* child */
		execvp("mount",arg);
		break;
	case -1:	/* error */
		break;
	default:	/* parent */
		wait(NULL);
		break;
	}
}

static void
desc2conf(struct config *conf,struct usb_device_descriptor desc)
{
	conf->id.class = desc.bDeviceClass;
	conf->id.subclass = desc.bDeviceSubClass;
	conf->id.protocol = desc.bDeviceProtocol;
	conf->id.vendor = desc.idVendor;
	conf->id.product = desc.idProduct;
	validate_config(conf);
}

#define	USB_DEV_NOCHANGE	0
#define	USB_DEV_NEW	1
#define	USB_DEV_DEL	2

/*
 * search nnn device in all devices
 * if no found return USB_DEV_NEW
 *	=> The config (which is a member in usbdevice) is set
 * else return USB_DEV_NOCHANGE
 */
int
read_proc_nnn(char *nnn,struct usbdevice *dev)
{
	struct usb_device_descriptor desc;
	int fd;

	if((fd = open(nnn,O_RDONLY)) == -1)
		return -1;
	if(read(fd,&desc,sizeof(struct usb_device_descriptor)) <= 0)
		return -1;
	close(fd);
	desc2conf(&(dev->conf),desc);
	if (find_device_all(*dev) == NULL)	/* not found */
		return USB_DEV_NEW;	/* new */

	return USB_DEV_NOCHANGE;
}


void
observ_proc_nnn(char *usb_dir)
{
	DIR *dp;
	struct dirent *de;
	struct usbdevice dev_tmp,*dev;
	struct config *conf;
	struct node *modlink;
	char path[PATH_MAX+1];

	if ((dp = opendir(usb_dir)) == NULL) {
		module_load(host_module);
		do_mount();
		if ((dp = opendir(usb_dir)) == NULL)
			return;
	}
	while((de = readdir(dp)) != NULL) {
		if (strlen(de->d_name) == 3 && isdigit(de->d_name[0]) &&
			isdigit(de->d_name[1]) && isdigit(de->d_name[2])) {
			if (debug)
				syslog(LOG_DEBUG,"\"%s\"\n",de->d_name);
			sprintf(path,"%s/%s",usb_dir,de->d_name);

			switch(read_proc_nnn(path,&dev_tmp)){
			case USB_DEV_NOCHANGE:
				if (debug)
					syslog(LOG_DEBUG,"no change");
				break;
			case USB_DEV_NEW:
				if (debug)
					syslog(LOG_DEBUG,"new device");

				/* todo NULL -> parent device */
				dev = create_device(NULL);
				dev->conf.id = dev_tmp.conf.id;
				/* todo: link device to modules */
				if ((conf = find_config_all(dev->conf)) == NULL) {
					if (debug)
						syslog(LOG_DEBUG,"not match config");
					/* todo no matched config */
					break;
				}
				if (debug)
					syslog(LOG_DEBUG,"match config");
				for (modlink = conf->link.data; ;modlink = modlink->next) {
					/* device links modlink and modlink links modules */
					create_modlink(&(dev->conf.link.data),modlink->data);
					module_load((char *)modlink->data->data);
					if (modlink->next == conf->link.data)
						break;
				}
				if (debug)
					syslog(LOG_DEBUG,"script=\"%s\"",dev->conf.script);
				/* execute starting script */
				if (conf->script != NULL)
					execute_script(conf->script);
				break;
			case USB_DEV_DEL:
				/* todo delete */
				break;
			}
		}
	}
	closedir(dp);
}

